/**************************************************************************
   Copyright (C) 1994 by R.Harmsen.
 *************************************************************************/

#include <stdlib.h>
#include <dos.h>
#include "timerchp.h"

/* Oscillator speed of the timer */
#define BASE_FREQ 1193180L

/********************************************************************
   The handler originally attached to the timer interrupt. To maintain
   time and date, this will still be called 18.2 times a second, even
   after speeding up the timer chip.
 *******************************************************************/
static time_handler *orig_time_handler = NULL;
static time_handler *toreset_time_handler = NULL;

/********************************************************************

 *******************************************************************/
time_handler *set_timer_interrupt
   (long rate, time_handler *new_time_handler)
{
   disable();
   if (orig_time_handler)
   {
      /* Must have been here before, before properly resetting.
         Do that first */
      reset_timer_interrupt();
   }
   set_timer_speed(rate);

   /* Hook a new handler to the timer interrupt, remember original */
   orig_time_handler = getvect(0x08);
   toreset_time_handler = new_time_handler;
   setvect(0x08, new_time_handler);
   enable();

   return orig_time_handler;
}


/********************************************************************
   Returns perhaps slightly modified rate.
 *******************************************************************/
long set_timer_speed (long rate)
{
   long divisor;

   /* Normalize to milli-seconds for easier transition to timing
      services with a minimum resolution of one milli-second,
      like MS-Windows multi-media facilities. But this has nasty
      consequences just now. */
   // rate = (long)(0.5 + 1000.0 / (long)(1000.0 / rate + 0.5));

   disable();
   /* Speed up the timer chip */
   /* Prepare chip for receipt of divisor value */
   /* Compare CTRL_REG in snd.h, only this is channel 0, not 2 */
   outp(0x43, 0x36);
   /* Send least- and then most significant byte of divisor */
   divisor = BASE_FREQ / rate;
   outp(0x40, (int)(divisor & 0xff));
   outp(0x40, (int)(divisor >> 8));
   enable();

   return rate;
}


/********************************************************************

 *******************************************************************/
void reset_timer_interrupt (void)
{
   long divisor;

   disable();
   if (!orig_time_handler)
   {
      enable();
      return;
   }

   /* Reset speed up of timer chip */
   outp(0x43, 0x36);
   divisor = 0x00;
   outp(0x40, (int)(divisor & 0xff));
   outp(0x40, (int)(divisor >> 8));

   /* Restore original handler */
   setvect(0x08, orig_time_handler);
   orig_time_handler = NULL;
   enable();
}


